------------------------------------------------------------------------------------------
-- Whammy
-- Created By:  Jaycyn of Hyjal
--
-- What is it: Heal, Decurse and more with a point and click
-- License   : Distribute as you wish, please give credit to the author, please send
--           :   bugs and suggestions instead of modifying code yourself.
------------------------------------------------------------------------------------------

--note that the namespace for Whammy is  Whammy = {} and defined in the localization file since it's the first to load

--some info
-- local constants per file are kConstantName
-- global constants are defined in this file  Whammy.constants.kConstantName
-- localization constants are defined in the WhammyLocalization.lua file  Whammy.localize.kConstantName  or  Whammy.localize.kcConstantName  or Whammy.localize.kmConstantName
--    k are used in this file, kc are used in WhammyConfig and km are in WhammyButton, which is the minimap button

--Define local constants
local kRed = 0; --Whammy text color
local kGreen = 1;
local kBlue = 0;
local kAlpha = .5;

local kButtonWidth = 90;
local kButtonHeight = 16;
local kFrameGutterTop = 10; --the top gutter of the whammy frame above the buttons. Used for dragging
local kFrameGutterRight = 10;
local kGutter = 2;
local kButtonWidthSmall = 45;
local kCurseHeight = 6;
local kUpdateInterval = 1.0; --used for range checking
local kFontNamePlayer = "GameFontGreenSmall";
local kFontNameOthers = "GameFontNormalSmall";
local kFontNameHighlight = "GameFontHighlightSmall";
local kFontNameDisable = "GameFontDisableSmall";

--define global variables used in this file
local gRaidList = {} --a list of raid1, raid2, raid3, used instead of concatenating raid..1  raid..2 etc for speed
local gPartyList = {}
for i = 1,4 do
	gPartyList = format("party%d", i)
end
for i = 1,40 do
	gRaidList = format("raid%d", i)
end

local gTimeSinceLastUpdate = 0; --used for range checking
local gIsSpellBookAvailable = false; -- this is set to true once player logs in so we can get spell data from the spellbook, avoids SPELL_CHANGED errors on first login
local gButtonWithThreat -- the button that is currently displaying threat

Whammy.buttonArray = {}; --keeps track of all the buttons in the frame
Whammy.unitIDArray = {}; --cross reference from the unit name stored in each button to the button the unit is stored in
Whammy.version = "Whammy" --placeholder until the addon loads and metadata can be read
Whammy.playerClass = nil --placeholder for player class. This is used when, for example, priest chakra changes


--global constants used throughout whammy files
Whammy.constants = {
	
	kButtonSizeStandard = 0,
	kButtonSizeSmall = 1,
	
	kPlayerAlone = 0,
	kPlayerInParty = 1,
	kPlayerInRaid = 2,
	
	kPriest = "Priest",
	-- kChakraSerenity = 88684, --holy word: Serenity
	-- kChakraSanctuary = 88685, --holy word: Sanctuary
	-- kChakraChastise = 88625 --Chastise
	
}

--used to handle changing priest 'stances' i.e. chakras
Whammy.Chakra = {}
Whammy.Chakra[1] = 88625 --Chastise
Whammy.Chakra[2] = 88685 --holy word: Sanctuary
Whammy.Chakra[3] = 88684 --holy word: Serenity
Whammy.Chakra[4] = 20711 --Spirit of Redemption


--craft the main whammy frame
local whammyFrame = CreateFrame("FRAME", "WhammyFrame", UIParent)
whammyFrame:RegisterEvent("ADDON_LOADED"); -- Fired when saved variables are loaded

whammyFrame:SetScript("OnEvent", 
    function(self, event, ...)
	   local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	   Whammy:Event_Handle( event, arg1, arg2); --tell the frame to send the events to the OnEvent function
	end)

function Whammy:Event_Handle(event, arg1, arg2)
	
	-- print("Whammy:Event_Handle:  "..event);
	-- 
	-- if ( arg1 ) then
	-- 	print("    arg1:  "..arg1)
	-- end
	-- 
	-- if ( arg2 ) then
	-- 	print("    arg2:  "..arg2)
	-- end


	if event == "ADDON_LOADED" and arg1 == "Whammy" then -- Our saved variables are ready at this point. If there are none, W_Options will be nil

		whammyFrame:UnregisterEvent("ADDON_LOADED"); --we dont need to know about other addons being loaded so unregister
		
		Whammy:Event_AddOnLoaded();
		
	elseif ( event =="PLAYER_LOGIN") then
		Whammy:Event_PlayerLogin();

	elseif ( event == "PLAYER_ENTERING_WORLD" ) then
		Whammy:Event_PlayerEnteringWorld();
		
	elseif ( event == "PLAYER_LEAVING_WORLD") then
		Whammy:Event_PlayerLeavingWorld();

	elseif ( event == "PLAYER_REGEN_DISABLED" ) then --we have entered combat
		Whammy:Event_InCombat( true );

	elseif ( event == "PLAYER_REGEN_ENABLED" ) then --we have left combat, so do whatever updates we couldn't do during combat
		Whammy:Event_InCombat( false );

	elseif ( event == "UNIT_THREAT_LIST_UPDATE") then
		Whammy:Event_UnitThreatListUpdate( arg1 ); --pass the mob whose threat list changed

	elseif ( event == "UNIT_THREAT_SITUATION_UPDATE" and W_Options.ThreatNotify ) then
		Whammy:Event_UnitThreatSituationUpdate( arg1 ); --pass the unit whose threat has moved on the mobs list

	elseif ( event == "PLAYER_TARGET_CHANGED" ) then
		Whammy:Event_PlayerTargetChanged( arg1 ); --arg1 doesn't matter

	elseif ( event == "GROUP_ROSTER_UPDATE") then -- the group has changed, players added or disbanded
		if ( InCombatLockdown() ) then
			whammyFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
		else
			Whammy:Event_GroupRosterUpdate();
		end

	elseif ( event == "UNIT_AURA" ) then --handle curses on raid or party members, ignore pets
		if ( UnitIsUnit( arg1, "player") or UnitInParty(arg1) or UnitInRaid(arg1) ) then --only if unit is the player or another player in the party or raid
			Whammy:Event_UnitAura(arg1); --arg1 us the unitID
		end

	elseif ( event == "UNIT_HEALTH" ) then
		Whammy:Event_UnitHealth(arg1); --arg1 is the unitID

	elseif ( event == "UNIT_POWER" ) then
		-- unitID - The unit that was affected. (string)
		-- powerType - 'MANA', 'RAGE', 'ENERGY', 'FOCUS', 'HAPPINESS', 'RUNIC_POWER', 'HOLY_POWER'. (string)
		
		local unitID = arg1
		-- local powerType = arg2
		
		if ( Whammy:IsUnitIDPlayerPartyRaid( unitID ) ) then --only update the whammysummary mana bar when the event is either this player or another player in party or raid
			
			WhammySummary:Event_UnitMana();
		end
		
	elseif ( event == "SPELLS_CHANGED" ) then

		Whammy:Event_SpellsChanged()
		
	-- elseif ( event == "LEARNED_SPELL_IN_TAB" ) then
	-- 	
	-- 	print("learned spell")
		
	end
end

-- 
-- --------
-- --------  EVENT HANDLERS --------
-- --------

function Whammy:Event_AddOnLoaded()
	--print("Event_AddOnLoaded")
	--set up the main whammy frame
	Whammy:Init_Options(); --init the options. Will set to defaults if there are none
	Whammy:FrameCreate();
	Whammy:FrameCreateUnitButtons(); --create the buttons
	Whammy:FrameCreateTestButtons(); --the buttons for testing
	Whammy.version = GetAddOnMetadata("Whammy", "Version");  --will be 0.0.7 xx/xx/xx
	
	local class, classFileName = UnitClass("player") --get this players class
	Whammy.playerClass = class
	
	
	-- set up the slash commands
	SLASH_WHAMMY1 = "/whammy";
	SLASH_WHAMMY2 = "/wham";
	--and the handler
	SlashCmdList["WHAMMY"] = function(msg)
		Whammy:Slash_Command_Handler(msg);
	end
	
	--set up the config frame
	WhammyConfig:Init();
	
	--these are the loading events upon first login or after zoning in/changing contintents
	whammyFrame:RegisterEvent("PLAYER_LOGIN");
	whammyFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
	--whammyFrame:RegisterEvent("SPELLS_CHANGED") --called when the spellbook is ready upon first login on when spells change in the book (chakra for example)
	
	--prepare for events
	whammyFrame:RegisterEvent("GROUP_ROSTER_UPDATE");
	whammyFrame:RegisterEvent("UNIT_AURA");
	whammyFrame:RegisterEvent("UNIT_HEALTH");
	whammyFrame:RegisterEvent("UNIT_POWER"); --unit_mana is depreciated
	whammyFrame:RegisterEvent("PLAYER_REGEN_DISABLED"); --use this to know if we enter or leave combat
	whammyFrame:RegisterEvent("PLAYER_ALIVE");
	--whammyFrame:RegisterEvent("UPDATE_SHAPESHIFT_FORM") --so we know when for example, a priest changes Chakra's
	--whammyFrame:RegisterEvent("LEARNED_SPELL_IN_TAB")
	
	--when transistioning continents, instances etc the spellbook may be in flux so ignore SPELLS_CHANGED events during that process
	whammyFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
	whammyFrame:RegisterEvent("PLAYER_LEAVING_WORLD")
	
	if ( W_Options.ShowWhammy ) then --if the last time the player played, Whammy was visible, keep it visible'
			whammyFrame:Show();
	end

	WhammyButton_Move(); --move the Whammy Minimap icon into position

	--Whammy_Test_Button:Show();
end

--create the options table if it doesnt exist and then add all of the required vars
function Whammy:Init_Options()
	--the saved variable per character that stores that chars settings and options between sessions (defined in TOC)
	
	if ( not W_Options ) then W_Options = {}; end
	
	if ( W_Options.Mute == nil ) then W_Options.Mute = true; end
	if ( W_Options.ShowDropdown == nil ) then W_Options.ShowDropdown = false; end
	if ( W_Options.Temp4 == nil ) then W_Options.Temp4 = false; end
	if ( W_Options.Temp5 == nil ) then W_Options.Temp5 = false; end
	if ( W_Options.ThreatNotify == nil ) then W_Options.ThreatNotify = true; end
	if ( W_Options.Movable == nil ) then W_Options.Movable = true; end
	if ( W_Options.ShowSummary == nil ) then W_Options.ShowSummary = true; end
	if ( W_Options.ShowWhammy == nil ) then W_Options.ShowWhammy = true; end
	if ( W_Options.ButtonSize == nil ) then W_Options.ButtonSize = Whammy.constants.kButtonSizeStandard; end
	if ( W_Options.Temp2 == nil ) then W_Options.Temp2 = false; end
	if ( W_Options.RangeCheck == nil ) then W_Options.RangeCheck = true; end
	if ( W_Options.Health1 == nil ) then W_Options.Health1 = "Off"; end
	if ( W_Options.Health2 == nil ) then W_Options.Health2 = "Off"; end
	if ( W_Options.Angle == nil ) then W_Options.Angle = 352; end
	if ( W_Options.Actions == nil ) then W_Options.Actions = {}; end --each row is a table with (item,spell or macro), ID, and either itemLink or spell/pet
	if ( W_Options.Actions2 == nil ) then W_Options.Actions2 = {}; end --the actions for secondary spec
	
end

-- This function is called from the main Whammy frame every W_UpdateInterval to check for range on the buttons. On Update is set in WhammyConfig
-- Note that it is enabled in the WhammyConfig section
function Whammy:Event_FrameOnUpdate(elapsed)
	gTimeSinceLastUpdate = gTimeSinceLastUpdate + elapsed;
	
	if ( gTimeSinceLastUpdate > kUpdateInterval ) then
		Whammy:UnitButtonRangeCheck();
		gTimeSinceLastUpdate = 0;
	end
	
end

--
-- This event signals that information on the user's spells and spellbook has been loaded and is available to the UI.
--   Also fires when something in the spellbook changes - chakra's for example
--
function Whammy:Event_SpellsChanged()
	--print("Event_SpellsChanged")

	WhammyConfig:DropSlot_Init(); --set up the dropslots with the spells from the saved variables per character
	
	-- -- dont try to fill the slots with spells until the spellbook is loaded.
	-- local i = GetNumSpellTabs()
	-- local okToLoadSlots = true;
	-- 
	-- --we need to wait to load the spells for a priest until the it form (chakra) is set
	-- if ( Whammy.playerClass == Whammy.constants.kPriest ) then -- if this is a priest
	-- 	local shapeShiftForm = GetShapeshiftForm()
	-- 	if ( shapeShiftForm == 0 ) then
	-- 		okToLoadSlots = false
	-- 	end
	-- end
	-- 
	-- if ( i > 0 and okToLoadSlots ) then
	-- 
	-- 	WhammyConfig:DropSlot_Init(); --set up the dropslots with the spells from the saved variables per character
	-- 	
	-- end

	
end

-- function Whammy:IsSpellBookAvailable()
-- 	
-- 	for i = 1,GetNumSpellTabs() do
-- 	  local name = GetSpellTabInfo(i);
-- 	  print(name);
-- 	end
-- 	
-- end

function Whammy:Event_PlayerLeavingWorld()

	whammyFrame:UnregisterEvent("SPELLS_CHANGED");
	whammyFrame:RegisterEvent("PLAYER_ENTERING_WORLD")

end

function Whammy:Event_PlayerLogin()
	--Whammy:ShowMessage("    Event_PlayerLogin")

	Whammy:FrameDrawUnitButtons(); --draw the buttons in their proper positions
	Whammy:AdjustDragBarSize()
	
	if ( not IsInGroup() ) then --if the player is not in a group set it as so, otherwise the group events will fire and set to group
		Whammy:SetGroupAsPlayer();
	end
	
	--at this point, talent information is available so register that event
	whammyFrame:RegisterEvent("PLAYER_TALENT_UPDATE");

end

--this event fires any time a player sees a loading screen
function Whammy:Event_PlayerEnteringWorld()

	whammyFrame:RegisterEvent("SPELLS_CHANGED");
	
end

function Whammy:Event_GroupRosterUpdate()
	--Whammy:ShowMessage("    Event_GroupRosterUpdate");
	
	Whammy:UnitButtonResetAll(); --clear out all the unit buttons as the group setup changed and we are re-loading them
	Whammy.unitIDArray = {} --reset this as it's about to be re-loaded
	
	local playerGroupStatus = Whammy:GetPlayerGroupStatus();
	
	if ( playerGroupStatus == Whammy.constants.kPlayerAlone ) then
		Whammy:SetGroupAsPlayer();
	elseif ( playerGroupStatus == Whammy.constants.kPlayerInParty ) then
		Whammy:SetGroupAsParty();
	elseif ( playerGroupStatus == Whammy.constants.kPlayerInRaid ) then
		Whammy:SetGroupAsRaid();
	else
		--should never get here
	end
end

function Whammy:SetGroupAsPlayer()
	--Whammy:ShowMessage("    player is alone");
	Whammy:UnitButtonSetName(1, "player"); --set button1 to player
	Whammy:RangeCheckToggle( false ); --no point in range checking myself
end

function Whammy:SetGroupAsParty()
	--Whammy:ShowMessage("    SetGroupAsParty")
	
	Whammy:UnitButtonSetName(1, "player"); --set button1 to player
			
	for i = 1,4 do --player is in pos #1, 1-4 are the other 4 members
		local unitID = "party"..i;
		local buttonNum = i+1; --button 1 is player, button2 is party1, button3 is party2 etc
		if ( UnitExists(unitID) ) then
			Whammy:UnitButtonSetName( buttonNum, unitID ); --party1 goes to button2, party2 goes to button3 etc. Also sets up the health amount for the unit
		else
			Whammy:UnitButtonReset(buttonNum);
		end
	end
	
	if ( W_Options.RangeCheck == true ) then
		WhammyConfig:RangeCheckEnable( true ); --we are in a party, start range checking if that option is enabled
	end
end

function Whammy:SetGroupAsRaid()
	--Whammy:ShowMessage("    SetGroupAsRaid");
	
	local unitCount, buttonNum, id, unitID;
	local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML;
	local subGroupUnitCount = {0,0,0,0,0,0,0,0}; --keep track of how many raid members are in each raid group. Starts at 0 (5 per group)
	
	local numGroupMembers = GetNumGroupMembers();
	--Whammy:ShowMessage ("numberGroupMembers "..numGroupMembers );
	
	for id = 1,MAX_RAID_MEMBERS do --this will run from 1 to whatever the maximum # of raid members is
		if ( id <= numGroupMembers ) then --this will only examine raid members 1...n, where n=number of actual raid members
			unitID = "raid"..id
			name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(id);	--get their name and subgroup
			subGroupUnitCount[ subgroup ] = subGroupUnitCount[ subgroup ] + 1; --keep track of how many units are in each subgroup
			unitCount = subGroupUnitCount[ subgroup ]; --get the next unit position based on the unit count of the subgroup
			buttonNum = ( ( (subgroup-1) * 5) + unitCount ) -- first subgroup, first position will be 1, 8th subgroup, first position will be 36
			Whammy:UnitButtonSetName( buttonNum, unitID ); --sets the name and background color
		end
	end
	
	if ( W_Options.RangeCheck == true ) then
		WhammyConfig.RangeCheckEnable( true ); --we are in a raid, start range checking if that option is enabled
	end
end

function Whammy:Event_PlayerTargetChanged( someArg )

	Whammy:UnitButtonClearThreat();
end

function Whammy:Event_UnitAura( unitID )

	local isRemovable = 1;
	local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellId, canApplyAura, isBossDebuff, value1, value2, value3;

	Whammy:UnitButtonSetAura( unitID, nil ); --clear the aura for this unitID button
	for debuffNum = 1,40 do
		name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellId, canApplyAura, isBossDebuff, value1, value2, value3 = UnitDebuff(unitID, debuffNum, isRemovable)
		if ( dispelType ) then -- if theres a debuff:  Magic Disease Poison Curse
			local alreadyHasAilment = Whammy:UnitButtonGetAura( unitID, dispelType); --see if this unit is already showing the ailment
			if ( alreadyHasAilment == false ) then
				Whammy:UnitButtonSetAura( unitID, dispelType ); -- indicate it on the unit button
				if ( W_Options.Mute == false ) then -- if we are not group, play a sound
					--Whammy:ShowMessage("UnitID = "..unitID.."  Aura = "..dispelType)
					PlaySoundFile("interface\\addons\\Whammy\\AilmentSound.mp3");
				end
			end
		end
	end
end

function Whammy:Event_UnitHealth( unitID )
	--Whammy:ShowMessage("Event_UnitHealth "..unitID)
	local healthFraction;
	local maxHealth = UnitHealthMax( unitID )
	local currentHealth = UnitHealth( unitID )
	
	if ( UnitIsConnected( unitID ) ) then
	
		if ( UnitIsDeadOrGhost( unitID ) or currentHealth < 1 ) then --unit dead
			healthFraction = 0;
		else
			healthFraction = currentHealth / maxHealth; --the % of how much health they have
			
			if (healthFraction < 0) then healthFraction = 0; end;
			if (healthFraction > 1) then healthFraction = 1; end;
		
			local healthAmount = healthFraction * 100;
			if ( W_Options.Health2 ~= "Off" and healthAmount < tonumber(W_Options.Health2) ) then
				Whammy:UnitButtonHealthAlertSet( unitID, 2, true)
			elseif ( W_Options.Health1 ~= "Off" and healthAmount < tonumber(W_Options.Health1) ) then
				Whammy:UnitButtonHealthAlertSet( unitID, 1, true)
			else
				Whammy:UnitButtonHealthAlertSet( unitID, nil, nil)
			end
		end
	
		Whammy:UnitButtonSetHealth( unitID, healthFraction ); --will adjust the texture of the button to indicate the amount of health

	end
end

function Whammy:Event_UnitThreatListUpdate( mobID ) --the Mob is the mob whose threat list changed or we just went into combat
	--Whammy:ShowMessage("Event_UnitThreatListUpdate "..mobID)
	--one way to check the mobID is to see who it's target it
	-- if ( UnitExists(mobID) ) then
	-- 
	-- 	if ( not UnitIsPlayer(mobID) and UnitCanAttack( "player", mobID) and UnitHealth( mobID) > 0 and not UnitPlayerControlled(mobID) ) then
	-- 		local mobTarget = mobID.."target"; --this is who has aggro on the mob
	-- 		local mobTargetGUID = UnitGUID(mobTarget); --the get tanks GUID
	-- 		if ( mobTargetGUID ) then
	-- 			--guidNameLookup[mobTargetGUID] = UnitName(mobTarget); --store it
	-- 			local tankName = UnitName( mobTarget );
	-- 			Whammy:ShowMessage("Threat List Update *** "..tankName.." *** HAS AGGRO");
	-- 		end
	-- 	
	-- 	end
	-- end

	--another way is to to check the raid to see who is firmly tanking the mob
	-- local aList;
	-- 
	-- if ( inParty or inRaid ) then
	-- 	if ( inRaid ) then
	-- 		aList = gRaidList; --array of raid1, raid2 etc
	-- 		currentPartySize = GetNumGroupMembers();
	-- 	else
	-- 		aList = gPartyList; --array of party1, party2 etc
	-- 		currentPartySize = GetNumGroupMembers();
	-- 	end
	-- 	
	-- 	for i = 1, currentPartySize do
	-- 		local unitID = aList[i];
	-- 		status = UnitThreatSituation( unitID, mobID)
	-- 		if ( status == 3 ) then --this unit has 100% threat
	-- 			local unitName = UnitName( unitID )
	-- 			Whammy:ShowMessage("*** "..unitName.." *** HAS AGGRO")
	-- 		end
	-- 	end
	-- 
	-- end
end

function Whammy:Event_UnitThreatSituationUpdate( unitID ) --the unit is the unit in the party or raid who changed threat position
	--unitID could be player, party1, raid1 etc... need to get the name
	local mobID = "playertarget"; --who is my target
	if ( unitID ) then
		if ( not UnitIsPlayer(mobID) and UnitCanAttack( "player", mobID) and UnitHealth( mobID) > 0 and not UnitPlayerControlled(mobID) ) then
		
			local mobName = UnitName( mobID );
			local uName = UnitName( unitID );
			if mobName and uName then --if we have a valid mob targeted and a valid unitID was passed in then check threat
				local status = UnitThreatSituation( unitID, mobID ); --if status = 3, the unitID has aggro
				--Whammy:ShowMessage("Event_UnitThreatSituationUpdate *** "..unitID.." *** has aggro on "..mobName);
				local msg = "*** "..uName.." *** has aggro on "..mobName
				UIErrorsFrame:AddMessage( msg, 1.0, 0.0, 0.0, 53, 5);
				local b = Whammy:GetButtonFromUnitID( unitID );
				if (b) then --if there is a button for this unit (i.e. they are in our raid or party)
					Whammy:UnitButtonSetThreat( b ); --set the threat indicator active, and turn it off on others
				end
			end
		 end
		
	end	

end

-- called when we enter or leave combat. Enable or disable the drop downs depending on combat status
function Whammy:Event_InCombat( isInCombat ) --isInCombat = true when entering combat, and false when leaving
	
	if ( isInCombat ) then --when in combat disable the growbox so we don't taint the frame and get an error, register the event to detect when we leave combat
		--Whammy:ShowMessage("in combat")
		whammyFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
		whammyFrame:RegisterEvent("UNIT_THREAT_LIST_UPDATE")
		whammyFrame:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
		whammyFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
		
		whammyFrame.sizer:SetScript("OnMouseDown", nil)
		whammyFrame.sizer:SetScript("OnMouseUp",   nil)
		
		WhammyConfig:HandleCombat();
	else --we have left combat, unlock frames, perform updates etc
		--Whammy:ShowMessage("left combat")
		whammyFrame:UnregisterEvent("PLAYER_REGEN_ENABLED") --already got the event so forget about it
		whammyFrame:UnregisterEvent("UNIT_THREAT_LIST_UPDATE")
		whammyFrame:UnregisterEvent("UNIT_THREAT_SITUATION_UPDATE")
		whammyFrame:UnregisterEvent("PLAYER_TARGET_CHANGED")
		
		Whammy:Event_GroupRosterUpdate();
		
		whammyFrame.sizer:SetScript("OnMouseDown", function( self ) Whammy:FrameGrowBoxOnMouseDown( self ); end)
		whammyFrame.sizer:SetScript("OnMouseUp",   function( self ) Whammy:FrameGrowBoxOnMouseUp( self ); end)
		Whammy:UnitButtonClearThreat();
		Whammy:FrameDrawUnitButtons();
		
		WhammyConfig:DropSlot_Init() --update the slots in case something changed during combat, stance, spell added etc.
	end
	
	Whammy:Event_InCombat_ShowBorder( isInCombat );

end

function Whammy:Event_InCombat_ShowBorder( isInCombat )

	if ( isInCombat ) then --show a faint red border around Whammy when in combat
		--Whammy:ShowMessage( "showing border")
		whammyFrame.topborder:Show();
		whammyFrame.rightborder:Show();
		whammyFrame.bottomborder:Show();
		whammyFrame.leftborder:Show();
	else --hide the border
		--Whammy:ShowMessage( "hiding border")
		whammyFrame.topborder:Hide();
		whammyFrame.rightborder:Hide();
		whammyFrame.bottomborder:Hide();
		whammyFrame.leftborder:Hide();
	end

end
-- --
-- --------  END EVENT HANDLERS --------
-- --
-- 
-- called once the variables are loaded - if they are nil, set to defaults


function Whammy:UnitButtonHealthAlertSet( unitID, whichAlert, isSet)
	local b = Whammy:GetButtonFromUnitID( unitID )
	local red, green, blue, alpha;
	if b then
		if ( isSet ) then
			if ( whichAlert == 1) then --yellow
				red=1; green=1; blue=0; alpha=1
			elseif ( whichAlert == 2 ) then --red
				red=1; green=0; blue=0; alpha=1
			end
			b.healthAlertLeftTexture:SetTexture( red, green, blue, alpha);
			b.healthAlertTopTexture:SetTexture( red, green, blue, alpha);
			b.healthAlertRightTexture:SetTexture( red, green, blue, alpha);
			b.healthAlertBottomTexture:SetTexture( red, green, blue, alpha);
		else
			b.healthAlertLeftTexture:SetTexture( nil );
			b.healthAlertTopTexture:SetTexture( nil );
			b.healthAlertRightTexture:SetTexture( nil );
			b.healthAlertBottomTexture:SetTexture( nil );
		end
	end
end

function Whammy:Test_Button_Click()
	--Whammy:ShowMessage("Test Button #1 click")
	--WhammyConfig:ShowGlobalArray();

	--Whammy:Test_ShowAllSpellsInSpellbook()
	--hammy:Test_ShowInfoAboutOneSpell()

	-- Whammy.Chakra[1] = 88625 --Chastise
	-- Whammy.Chakra[2] = 88685 --holy word: Sanctuary
	-- Whammy.Chakra[3] = 88684 --holy word: Serenity
	-- Whammy.Chakra[4] = 20711 --Spirit of Redemption

	local globalSpellID = Whammy.Chakra[1]

	local spellName, rank, icon, castingTime, minRange, maxRange, spellID = GetSpellInfo(globalSpellID)
	
	print(spellName)
	print(spellID)
	
	ClearCursor()
	PickupItem(spellName)
	
	
	--local shapeShiftIndex = GetShapeshiftForm()  --what form are we currently, for priests, 1-4

	
	-- local keyPress = ""
	-- local theType = "spell"
	-- local actionWhenPressed = spellName
	-- local mouseButton = 1
	-- 
	-- if ( keyPress == "ctrl" or keyPress == "shift" or keyPress == "alt") then --if there is a keypress modifier then we need to add 
	-- 	keyPress = ( keyPress.."-" ); --                                       the dash so the attribute will be properly formatted
	-- end
	-- 
	-- local x = (keyPress.."type"..mouseButton);
	-- local y = (keyPress..theType..mouseButton);
	-- 
	-- for buttonNum = 1,MAX_RAID_MEMBERS do
	-- 	local b = Whammy.buttonArray[buttonNum];
	-- 
	-- 	b:SetAttribute( x, theType); --this attribute is the type spell, item etc.  paramters would be:  shifttype1, spell
	-- 	b:SetAttribute( y, actionWhenPressed); --this is the thing to perform on.  shiftspell1, Arcane Intellect
	-- end
	


end

function Whammy:Test_ShowInfoAboutOneSpell()
	
	local holyWordSerenity = 88684 --holy word: Serenity
	local holyWordSanctuary = 88625
	local thing = 32546 --binding heal
	local ser = "Holy Word: Serenity"
	local sanc = "Holy Word: Sanctuary"
	
	Whammy:Test_ShowAllSpellsInSpellbook()
	
	-- local spellName, rank, icon, castingTime, minRange, maxRange, globalSpellID = GetSpellInfo(index, "bookType") or GetSpellInfo("name") or GetSpellInfo(id)
	--local Name, Subtext = GetSpellBookItemName(index, "bookType") or GetSpellBookItemName("spellName")
	
	-- print("--  FINDING SPELL --")
	-- 
	-- local index = Whammy:GetSpellBookIndex( globalSpellID );
	-- 
	-- -- local spellName, rank, icon, castingTime, minRange, maxRange, globalSpellID2 = GetSpellInfo(globalSpellID)
	-- -- print( " found spell: "..spellName)
	-- 
	-- if ( index ) then
	-- 	print(" picking up")
	-- 	PickupSpellBookItem(index, BOOKTYPE_SPELL)
	-- else
	-- 	print(" index was nil")
	-- end
	
	-- local isSpellInSpellbook = Whammy:Test_IsSpellKnown(globalSpellID)
	-- local slot = FindSpellBookSlotBySpellID(globalSpellID)
	-- 
	-- if ( isSpellInSpellbook == true) then
	-- 	print("--> yes")
	-- 	print("spell in slot "..slot)
	-- else
	-- 	print("--> no")
	-- end
	-- 
	-- 
	-- local spellName, rank, icon, castingTime, minRange, maxRange, globalSpellID2 = GetSpellInfo(globalSpellID)
	-- print("spellName = "..spellName)
	-- print("descrip   = "..GetSpellDescription(globalSpellID) )
	-- print("globalSpellID2 "..globalSpellID2)
	-- 
	-- local spellName2, subText = GetSpellBookItemName(spellName)
	-- print("spellName2 = "..spellName2)
	-- print("subtext    = "..subText)
	-- 
	-- local skillType, spellId2 = GetSpellBookItemInfo(spellName)
	-- print("skillType = "..skillType)
	-- print("spellId2  = "..spellId2)
end

function Whammy:Test_IsSpellKnown(spellID)
	local slot = FindSpellBookSlotBySpellID(spellID)
	if slot and slot <= MAX_SPELLS then
		local slotType, id = GetSpellBookItemInfo(slot, BOOKTYPE_SPELL)
		return id == spellID and slotType ~= 'FUTURESPELL'
	end
end

function Whammy:Test_ShowAllSpellsInSpellbook()

	local i = 1
	while true do
	   local spellName, spellSubName = GetSpellBookItemName(i, BOOKTYPE_SPELL)
	   if not spellName then
	      do break end
	   end

	   -- use spellName and spellSubName here
	   DEFAULT_CHAT_FRAME:AddMessage( spellName .. '(' .. spellSubName .. ')' )

	   i = i + 1
	end

	
end


function Whammy:Test_Button2_Click()
	--Whammy:ShowMessage("Test Button #2 click")
	
end

function Whammy:FrameCreate()
	
	whammyFrame:SetFrameStrata("LOW")
	whammyFrame:SetClampedToScreen(true)

	whammyFrame:SetSize(380, 200)
	whammyFrame:SetPoint("CENTER", 0, 0)
	whammyFrame:SetResizable(true)
	whammyFrame:SetMaxResize( UIParent:GetWidth(), UIParent:GetHeight() - 50)
	whammyFrame:EnableMouse(true)
	whammyFrame:SetMovable(true)
	
	--set up a title region in the frame used for dragging
 	whammyFrame.titleRegion = whammyFrame:CreateTitleRegion()
	whammyFrame.titleRegion:SetSize(380, 10)
	whammyFrame.titleRegion:SetPoint("TOPLEFT", whammyFrame, "TOPLEFT")
	--this is the drag bar texture
	whammyFrame.titleTexture = whammyFrame:CreateTexture("WhammyTitleTexture", "ARTWORK")
	whammyFrame.titleTexture:SetSize(380, 10)
	whammyFrame.titleTexture:SetPoint("TOPLEFT")
	whammyFrame.titleTexture:SetTexture( .3, .3 , .3, .5)
	
	--background texture for the entire frame
	whammyFrame.texture = whammyFrame:CreateTexture("WhammyFrameBackground_texture", "ARTWORK")
	whammyFrame.texture:SetTexture(.1,.1,.1,.4)
	whammyFrame.texture:SetAllPoints(whammyFrame)

	--the sizer widger in the lower right corner of the frame
	whammyFrame.sizer = CreateFrame("Frame", "WhammyFrame_Sizer", whammyFrame)
	whammyFrame.sizer:SetWidth(16)
	whammyFrame.sizer:SetHeight(16)
	whammyFrame.sizer:SetPoint("BOTTOMRIGHT", whammyFrame, "BOTTOMRIGHT", 0, 0)
	whammyFrame.sizer.texture = whammyFrame.sizer:CreateTexture("myframe_sizer_texture", "ARTWORK")
	whammyFrame.sizer.texture:SetAllPoints( whammyFrame.sizer )
	whammyFrame.sizer.texture:SetTexture("Interface\\AddOns\\Whammy\\Images\\FrameResizeIcon")
	whammyFrame.sizer:EnableMouse();
	whammyFrame.sizer:SetScript("OnMouseDown", function( self ) Whammy:FrameGrowBoxOnMouseDown( self ); end)
	whammyFrame.sizer:SetScript("OnMouseUp",   function( self ) Whammy:FrameGrowBoxOnMouseUp( self ); end)

	--this is the red border that's displays when you are in combat
	whammyFrame.topborder = whammyFrame:CreateTexture("top_border", "BACKGROUND");
	whammyFrame.topborder:SetTexture(0.6, 0, 0, 0.4)
	whammyFrame.topborder:SetPoint("TOPLEFT", whammyFrame, "TOPLEFT", -1, 1);
	whammyFrame.topborder:SetPoint("BOTTOMRIGHT", whammyFrame, "TOPRIGHT", 0, 0);

	whammyFrame.rightborder = whammyFrame:CreateTexture("right_border", "BACKGROUND");
	whammyFrame.rightborder:SetTexture(0.6, 0, 0, 0.4)
	whammyFrame.rightborder:SetPoint("TOPLEFT", whammyFrame, "TOPRIGHT", 0, 1);
	whammyFrame.rightborder:SetPoint("BOTTOMRIGHT", whammyFrame, "BOTTOMRIGHT", 1, -1);

	whammyFrame.bottomborder = whammyFrame:CreateTexture("bottom_border", "BACKGROUND");
	whammyFrame.bottomborder:SetTexture(0.6, 0, 0, 0.4)
	whammyFrame.bottomborder:SetPoint("TOPLEFT", whammyFrame, "BOTTOMLEFT", -1, -1);
	whammyFrame.bottomborder:SetPoint("BOTTOMRIGHT", whammyFrame, "BOTTOMRIGHT", 1, -2);

	whammyFrame.leftborder = whammyFrame:CreateTexture("left_border", "BACKGROUND");
	whammyFrame.leftborder:SetTexture(0.6, 0, 0, 0.4)
	whammyFrame.leftborder:SetPoint("TOPLEFT", whammyFrame, "TOPLEFT", -2, 1);
	whammyFrame.leftborder:SetPoint("BOTTOMRIGHT", whammyFrame, "BOTTOMLEFT", -1, -2);

	whammyFrame.topborder:Hide();
	whammyFrame.rightborder:Hide();
	whammyFrame.bottomborder:Hide();
	whammyFrame.leftborder:Hide();
	
end

function Whammy:FrameCreateUnitButtons()

	local strata = "ARTWORK"
	
	for i = 1,MAX_RAID_MEMBERS do
		local bName = "Whammy_Button"..i;
		local b = CreateFrame("Button", bName, whammyFrame, "WhammyUnitButtonTemplate");		--the button frame
		tinsert(Whammy.buttonArray, b); --insert the button into the button array
		b:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp", "Button4Up", "Button5Up"); -- the buttons need to see the mouse clicks
		b:SetWidth(kButtonWidth);
		b:SetHeight(kButtonHeight);
		-- the texture which adjusts depending on health
		b.texture = b:CreateTexture(bName.."_Texture", "BACKGROUND");
		b.texture:SetAllPoints(b);
		b.texture:SetTexture( 1, 1, 1, .4 );
		b:Disable();
		
		-- the unit name
		b.unitName = b:CreateFontString( bName.."_UnitName", strata, kFontNameOthers )
		b.unitName:SetPoint("CENTER", b, "CENTER", 0, 0);

		--the alert indicator for low health warnings 1 and 2
		b.healthAlertLeftTexture = b:CreateTexture( bName.."HealthAlert_Left_Texture", strata);
		b.healthAlertLeftTexture:SetPoint( "TOPLEFT", b, "TOPLEFT", 0, 0 );
		b.healthAlertLeftTexture:SetPoint( "BOTTOMRIGHT", b, "BOTTOMLEFT", 2, 0);
		b.healthAlertTopTexture = b:CreateTexture( bName.."HealthAlert_Top_Texture", strata);
		b.healthAlertTopTexture:SetPoint( "TOPLEFT", b, "TOPLEFT", 0, 0 );
		b.healthAlertTopTexture:SetPoint( "BOTTOMRIGHT", b, "TOPRIGHT", 0, -2);
		b.healthAlertRightTexture = b:CreateTexture( bName.."HealthAlert_Right_Texture", strata);
		b.healthAlertRightTexture:SetPoint( "TOPRIGHT", b, "TOPRIGHT", 0, 0 );
		b.healthAlertRightTexture:SetPoint( "BOTTOMLEFT", b, "BOTTOMRIGHT", -2, 0);
		b.healthAlertBottomTexture = b:CreateTexture( bName.."HealthAlert_Bottom_Texture", strata);
		b.healthAlertBottomTexture:SetPoint( "BOTTOMLEFT", b, "BOTTOMLEFT", 0, 0 );
		b.healthAlertBottomTexture:SetPoint( "TOPRIGHT", b, "BOTTOMRIGHT", 0, 2);
			
		--When the unit gets a curse (or other ailment) the top lights up in yelloWhammy: Create those areas
		b.curseTexture = b:CreateTexture(bName.."_CurseTop", strata)
		b.curseTexture:SetPoint( "TOPLEFT", b, "TOPLEFT", 0, 0 );
		b.curseTexture:SetPoint( "BOTTOMRIGHT", b, "TOPRIGHT", 0, -4);
		
		--When in a party or raid, and a unit gets aggro, display something on the button to indicate they have aggro
		b.threatTexture = b:CreateTexture( bName.."_Threat", strata)
		b.threatTexture:SetPoint( "TOPRIGHT", b, "RIGHT", 0, 2 );
		b.threatTexture:SetPoint( "BOTTOMLEFT", b, "RIGHT", -8, -2);
		
		--When the unit gets a curse( or other ailment ), there are letters on the curse bar that indicate what it is: M)agic C)urse D)isease P)oison
		local y = 4;
		b.magicText = b:CreateFontString( (bName.."_Curse_TextM"), "OVERLAY", "GameFontNormalSmall" )
		b.magicText:SetPoint( "TOPRIGHT", b, "TOPRIGHT", -27, y)
		b.magicText:SetTextColor( 1, 0, 0 );
		b.curseText = b:CreateFontString( (bName.."_Curse_TextC"), "OVERLAY", "GameFontNormalSmall" )
		b.curseText:SetPoint( "TOPRIGHT", b, "TOPRIGHT", -17, y)
		b.curseText:SetTextColor( 1, 0, 0 );
		b.diseaseText = b:CreateFontString( (bName.."_Curse_TextD"), "OVERLAY", "GameFontNormalSmall" )
		b.diseaseText:SetPoint( "TOPRIGHT", b, "TOPRIGHT", -9, y)
		b.diseaseText:SetTextColor( 1, 0, 0 );
		b.poisonText = b:CreateFontString( (bName.."_Curse_TextP"), "OVERLAY", "GameFontNormalSmall" )
		b.poisonText:SetPoint( "TOPRIGHT", b, "TOPRIGHT", 1, y)
		b.poisonText:SetTextColor( 1, 0, 0 );
		-- this is the little Rng that appears on each unit button when the unit is out of range
		b.rangeText = b:CreateFontString( bName.."_RangeText", "OVERLAY", "GameFontNormalSmall");
		b.rangeText:SetPoint("BOTTOMRIGHT", b, "BOTTOMRIGHT", 1, -2);
		b.rangeText:SetText("");
		b.rangeText:SetTextHeight(10);
		b.rangeText:SetTextColor(1, .5, 1)
	end
	--Whammy:ShowMessage("Whammy:FrameCreateUnitButtons complete")
end

function Whammy:FrameGrowBoxOnMouseDown( theGrowBox )
	theGrowBox:GetParent():StartSizing("BOTTOMRIGHT")
end

function Whammy:FrameGrowBoxOnMouseUp( theGrowBox )
	local parentFrame = theGrowBox:GetParent();
	parentFrame:StopMovingOrSizing()
	Whammy:FrameDrawUnitButtons();
	Whammy:AdjustDragBarSize();

end

function Whammy:AdjustDragBarSize()
	
	--update the drag bar title area width
	local w = whammyFrame:GetWidth();
	whammyFrame.titleRegion:SetWidth( w )
	whammyFrame.titleTexture:SetWidth( w )
	
end

-- function Whammy:MouseDown( self, ... )
-- 	local theMouseButton = ...;
-- 	
-- 	if ( theMouseButton == "LeftButton" ) then --if we clicked the left button
-- 		if ( W_Options.Movable ) then --and moving is enabled
-- 			self:StartMoving(); --then move the frame
-- 		end
-- 	elseif ( theMouseButton == "RightButton" and ( IsLeftShiftKeyDown() or IsRightShiftKeyDown() )) then --if right click + shift, let the user lock/unlock the frame
-- 		--present a popup list to lock or unlock the frame
-- 		Whammy:FrameMenuIsFrameMovable();
-- 	end
-- end
-- 
-- function Whammy:MouseUp( self )
-- 	self:StopMovingOrSizing()
-- end

-- function Whammy:FrameMenuIsFrameMovable() --display a popup to allow the Whammy Frame to be set to Movable or not
-- 	local menuFrame = CreateFrame("Frame", "ExampleMenuFrame", UIParent, "UIDropDownMenuTemplate")
-- 
-- 	local function onOption1()
-- 		W_Options.Movable = true;
-- 		if (WhammyConfigFrame) then
-- 			WhammyConfigFrame.isFrameMovableButton:SetChecked( W_Options.Movable );
-- 		end
-- 	end
-- 	local function onOption2()
-- 		W_Options.Movable = false;
-- 		if (WhammyConfigFrame) then
-- 			WhammyConfigFrame.isFrameMovableButton:SetChecked( W_Options.Movable );
-- 		end
-- 	end
-- 
-- 	local menuList = {
-- 	  { text = "Movable", func = onOption1 },
-- 	  { text = "Locked", func = onOption2 }
-- 	}
-- 
-- 	EasyMenu(menuList, menuFrame, "cursor", 0 , 0, "MENU");
-- end

function Whammy:FrameDrawUnitButtons()
	--Whammy:ShowMessage("Whammy:FrameDrawUnitButtons")
	local bHeight = kButtonHeight + kGutter
	local bWidth, w;
	local frameWidth = whammyFrame:GetWidth();
	local frameHeight = whammyFrame:GetHeight();
	local rightLimit = frameWidth - kFrameGutterRight; --we start drawing at 0, not bWidth so we need to compensate for that
	local x = 0;
	local y = -10;
	local col = 0;

	if ( W_Options.ButtonSize == Whammy.constants.kButtonSizeStandard ) then
		w = kButtonWidth
		bWidth = w + kGutter;
	else
		w = kButtonWidthSmall
		bWidth = w + kGutter;
	end
	--sets the min size of the whammyframe depending on the size of the buttons.
	whammyFrame:SetMinResize( w + kGutter + kFrameGutterRight, kButtonHeight + 3 + kFrameGutterTop)

	--draw the buttons
	for buttonNum = 1,MAX_RAID_MEMBERS do
		local b = Whammy.buttonArray[buttonNum];
		b:SetWidth( w );
		Whammy:UnitButtonSetNameWidth( b );
		
		x = col * bWidth;
		
		if ( x > rightLimit or (x+bWidth) > rightLimit ) then --if this button would go past the right side of the frame, drop it down a line
			x = 0;
			col = 0;
			y = y - bHeight
		end
		
		if ( -y > frameHeight or (-y+bHeight) > frameHeight ) then
			b:Hide()
		else
			b:Show()
		end
		
		b:ClearAllPoints()
		b:SetPoint("TOPLEFT", whammyFrame, "TOPLEFT", x, y)
		col = col + 1;
	end
end

function Whammy:FrameCreateTestButtons()
	local testButton1 = CreateFrame("Button", "Whammy_Test_Button", whammyFrame, "UIPanelButtonTemplate")
	testButton1:SetWidth(80);
	testButton1:SetHeight(22);
	testButton1:SetPoint("TOPLEFT", whammyFrame, "BOTTOMLEFT", 5, 5);
	testButton1:SetScript("OnClick", function() Whammy:Test_Button_Click(); end);
	testButton1:SetText("Test1");
	testButton1:Hide()

	local testButton2 = CreateFrame("Button", "Whammy_Test_Button2", whammyFrame, "UIPanelButtonTemplate")
	testButton2:SetWidth(80);
	testButton2:SetHeight(22);
	testButton2:SetPoint("RIGHT", whammyFrame, "LEFT", 5, 5)
	testButton2:SetScript("OnClick", function() Whammy:Test_Button2_Click(); end);
	testButton2:SetText("Test2");
	testButton2:Hide()
end

function Whammy:TestBoolean( someB )
	if (someB == 1) then
		return "true"
	elseif (someB == 0) then
		return "false"
	else
		return "nil"
	end
end

function Whammy:UnitButtonReset( i )
	--Whammy:ShowMessage("Whammy:UnitButtonReset");
	local b = Whammy.buttonArray[i];	

	--reset the buttons and cancel watching for clicks.
	b:SetScript("OnMouseDown", nil ); --thebutton and the mousebutton
	b:SetScript("OnMouseUp", nil);

	b.unitName:SetText(nil)

	b.texture:SetTexture( 1, 1, 1, .4 ); --sets the background to default
	b.texture:SetAllPoints( b ); --sets the texture to full size (health)

	b.magicText:SetText(nil); --clear out any aura's
	b.curseText:SetText(nil);
	b.diseaseText:SetText(nil);
	b.poisonText:SetText(nil);

	b.curseTexture:SetTexture( nil ); --clear the curse bar

	b.rangeText:SetText(nil); --clear out the range text

	b.unitName:SetFontObject(kFontNameOthers); --reset the font

	b.healthAlertLeftTexture:SetTexture( nil ); --reset the healh alert indicators
	b.healthAlertTopTexture:SetTexture( nil );
	b.healthAlertRightTexture:SetTexture( nil );
	b.healthAlertBottomTexture:SetTexture( nil );

	local unitID = b:GetAttribute("unit"); --get the unitID if there was one
	if (unitID) then --if the array that links unitID's to buttons had an entry for this, then remove it
		Whammy.unitIDArray[unitID] = nil; --removes the unitID entry from the dictionary
	end
	b:SetAttribute("type", "stop"); --set the function of the button to nothing
	b:SetAttribute("unit", nil)

	b:Disable(); --disable the button

end

function Whammy:UnitButtonResetAll()
	for i = 1,MAX_RAID_MEMBERS do
		Whammy:UnitButtonReset(i)
	end
end

function Whammy:UnitButtonSetAura( unitID, theAura) --when theAura=nil, then clear the debuff spot
	local b = Whammy:GetButtonFromUnitID( unitID )

	if (b) then --there could be a time when, you are in combat, and added someone to the party/raid that b could be nil
		if ( theAura ) then --there is a debuff, figure out what it is and display the text
			--Whammy:ShowMessage("Setting "..theAura);
			if (theAura == "Magic") then --if it was magic then set the magic text otherwise clear it
				b.magicText:SetText(Whammy.localize.kMagicText);
			elseif (theAura == "Curse") then
				b.curseText:SetText(Whammy.localize.kCurseText);
			elseif (theAura == "Disease") then
				b.diseaseText:SetText(Whammy.localize.kDiseaseText);
			elseif (theAura == "Poison") then
				b.poisonText:SetText(Whammy.localize.kPoisonText);
			end
			
			local result = b.curseTexture:SetTexture( .97, 1, .38, 1)--yellow bar across the top of the button
			--don't need to worry about result at this time
		else
			b.magicText:SetText(nil); --if there are no debuffs, clear them all
			b.curseText:SetText(nil);
			b.diseaseText:SetText(nil);
			b.poisonText:SetText(nil);
			
			b.curseTexture:SetTexture( nil )
		end	
	end
end

function Whammy:UnitButtonGetAura( unitID, theAura)  -- determine if this unit already has the specified ailment, return true if so
	local b = Whammy:GetButtonFromUnitID( unitID )
	local theText;
	
	if (b) then
		if (theAura == "Magic") then
			theText = b.magicText:GetText();
			
		elseif (theAura == "Curse") then
			theText = b.curseText:GetText();
			
		elseif (theAura == "Disease") then
			theText = b.diseaseText:GetText();
			
		elseif (theAura == "Poison") then
			theText = b.poisonText:GetText();
			
		end
	end
	
	if ( theText ) then
		return true;
	else
		return false;
	end
end

function Whammy:UnitButtonSetHealth( unitID, healthAmountInDecimal )
	--Whammy:ShowMessage("UnitButtonSetHealth");
	local buttonWidth;
	
	local b = Whammy:GetButtonFromUnitID( unitID );--get the button to change based on the unitID whose health is changing
	if b then
		if ( healthAmountInDecimal > 0 ) then -- unit is alive so set up their health
			if ( W_Options.ButtonSize == Whammy.constants.kButtonSizeStandard ) then
				buttonWidth = kButtonWidth;
			else
				buttonWidth = kButtonWidthSmall;
			end
			Whammy:UnitButtonSetClassColorAndStatus( b, unitID ); --check the range and set the colors
			local delta = buttonWidth - (buttonWidth * healthAmountInDecimal); --if w=100 and health=100%, then this would be 100-(100*1) = 0
			b.texture:SetPoint("TOPLEFT", b ,"TOPLEFT", 0, 0);
			b.texture:SetPoint("BOTTOMRIGHT", b, "BOTTOMRIGHT", -delta, 0);
		else --they are dead, dim the texture
			Whammy:UnitButtonSetAura( unitID, nil ) --reset the aura's and curse bar
			Whammy:UnitButtonSetClassColorAndStatus( b, unitID ); --check the range and set the colors
		end
	-- else
	-- 	Whammy:ShowMessage("unitButtonSetHealth, b was nil")
	end
end

function Whammy:UnitButtonSetName(whichButtonNum, unitID)
	--Whammy:ShowMessage("Whammy:UnitButtonSetName:  buttonNum = "..whichButtonNum.."  unitID = "..unitID);
	--whichButtonNum is the button number 1-40 and the unitID will be raid1, raid2 etc

	local b = Whammy.buttonArray[whichButtonNum];
	
	--since it has a unit, accept clicks and handle them
	b:SetScript("OnMouseDown", function(self, arg1) Whammy:UnitButtonMouseDown(self, arg1); end); --thebutton and the mousebutton
	b:SetScript("OnMouseUp", function(self) Whammy:UnitButtonMouseUp(self); end);
	
	Whammy.unitIDArray[unitID] = b; --associate a unitID table with the button so we can look up buttons via a unitID

	b.unitName:SetText( UnitName( unitID ) );
	Whammy:UnitButtonSetNameWidth( b );
	b:SetAttribute("unit", unitID); --set the assignment of the button to the passed in unit
	Whammy:UnitButtonSetClassColorAndStatus( b, unitID );
	Whammy:Event_UnitHealth(unitID); --set the initial health for the unit
	
	if ( UnitIsUnit( unitID, "player") ) then --if they are online, color the player one way, and the rest of the units another
		b.unitName:SetFontObject( kFontNamePlayer )
	else --someone else in the party or raid
		b.unitName:SetFontObject( kFontNameOthers )
	end
	
	b:Enable();
end

function Whammy:UnitButtonRangeCheck() 
	for buttonNum = 1,MAX_RAID_MEMBERS do
		
		local b = Whammy.buttonArray[buttonNum];
		local unitID = b:GetAttribute("unit");

		if (unitID) then
			Whammy:UnitButtonSetClassColorAndStatus( b, unitID );
		end
		
	end
end

-- called when assigning a unit to a button. Determine what kind class of unit (warrior, mage etc) and color the backgroud to match
-- also called from the range checking to dim the button
function Whammy:UnitButtonSetClassColorAndStatus(b, unitID)
	local class, classFileName = UnitClass(unitID); --get the type of class and a path to it's file, which I am not using at the moment
	local color = RAID_CLASS_COLORS[classFileName]; -- get the standard class colors in a color.r, color.g, color.b format
	local red, green, blue, alpha, d;
	local isPlayer = UnitIsUnit(unitID, "player");
	local inRange, checkedRange = UnitInRange(unitID);
	local isDead = UnitIsDead(unitID);
	local isGhost = UnitIsGhost(unitID);
	local isConnected = UnitIsConnected(unitID);
	
	if ( class ) then
		if ( isDead ) then --if dead clear to black, ghost clear to grey
			b.texture:SetTexture( 0, 0, 0, 1 ); --set to black when dead
			if ( inRange or isPlayer ) then --if in range or player, clear range text
				b.rangeText:SetText( "" );
			else --out or range or not player, show range text
				b.rangeText:SetText( Whammy.localize.kOutOfRangeText );
			end

		elseif (isGhost) then
			b.texture:SetTexture( 0.3, 0.3, 0.3, 0.9 );
			if ( inRange or isPlayer ) then --if in range or player, clear range text
				b.rangeText:SetText( "" );
			else --out or range or not player, show range text
				b.rangeText:SetText( Whammy.localize.kOutOfRangeText );
			end

		elseif (isConnected == nil) then --disconnected, set to black and a disc label
			b.texture:SetTexture( 0, 0, 0, 1 )
			b.rangeText:SetText( Whammy.localize.kDisconnectedText );
		
		else --they are alive, the are either in range or out of range
			if (isPlayer) then --if the unit is this player, then full color and clear the disc or rng txt
				b.texture:SetTexture( color.r, color.g, color.b, 1 )
				b.rangeText:SetText( "" );
			else
				if ( inRange ) then
					d = 1;
					alpha = 1;
					b.rangeText:SetText( "" );
				else
					d = .8;
					alpha = .7
					b.rangeText:SetText( Whammy.localize.kOutOfRangeText );
				end
	
				red = color.r * d;
				green = color.g * d;
				blue = color.b * d;
				b.texture:SetTexture( red, green, blue, alpha )
			end
		end
	end
end

--called when we assign a new unit to a unitbutton, to make their name fit in the width of the button, or when we change button size
function Whammy:UnitButtonSetNameWidth( b )
	
	local buttonWidth = b:GetWidth(); --the width of the button
	local stringWidth = b.unitName:GetStringWidth(); --the width of the unit name

	if ( stringWidth > buttonWidth ) then --if the unit name is wider than the button, then truncate the unit name
		local newStringWidth = buttonWidth * .95 -- the new string width will be 95% of the button width
		b.unitName:SetWidth( newStringWidth );
	else --the unit name fit within the box so reset the string width value
		b.unitName:SetWidth( buttonWidth );
	end
end

function Whammy:UnitButtonSetThreat( b )
	--Whammy:ShowMessage("UnitButtonSetThreat "..b:GetName() )
	
	Whammy:UnitButtonClearThreat();
	
	b.threatTexture:SetTexture( 0, 0, 0, 1)-- indicate this button has threat
	gButtonWithThreat = b; --keep track of which one this is

end


--
-- Handle Unit Button Mouse
--
function Whammy:UnitButtonMouseDown( b, mouseButton) --MouseDown and up highlight the unit name when clicked. arg1 is the LeftButton, etc. unused
	--Whammy:ShowMessage("UnitButton_MouseDown "..b:GetName().."  "..mouseButton);
	--mousebutton will be LeftButton, MiddleButton, RightButton, Button4, Button5
	
	if ( b:IsEnabled() ) then
		local unitName = b.unitName:GetText();
		b.unitName:SetFontObject(kFontNameHighlight)
		if ( Whammy:UnitButtonCheckForAction( b, mouseButton ) ~= true ) then --if there are no actions for this unit button, then see if we want to display a dropdown
			if ( W_Options.ShowDropdown ) then --if we want to show the drop down, validate unit and show it
				Whammy:UnitButtonShowMenu( b );
			end
		end
	end
end

function Whammy:UnitButtonShowMenu( b ) --if the button has a unit assigned but nothing in the mouse button action then show a menu
	--Whammy:ShowMessage("     showing dropdown");
	local menuFrame = CreateFrame("Frame", "W_UnitButtonDropDown", b, "UIDropDownMenuTemplate")
	local text, func, menuList;
	local unitID = b:GetAttribute("unit"); --which unitID is associated with this button
	local isInspectDisabled, isTradeDisabled, isChatDisabled;
	
	if ( not UnitIsUnit("player", unitID)  ) then --only open a drop down menu if we are clicking on a unit button that is not "player"
		if ( CheckInteractDistance( unitID, 1) ) then isInspectDisabled = 1; end;
		if ( CheckInteractDistance( unitID, 2) ) then isTradeDisabled = 2; end;
		if ( CheckInteractDistance( unitID, 4) ) then isFollowDisabled = 1; end;
		
		local menuList = {
			{
				text = Whammy.localize.kUnitButtonMenuTrade,
				func = function() Whammy:UnitButtonHandleMenu("trade", unitID) end,
				disabled = isTradeDisabled,
			},
			{
				text = Whammy.localize.kUnitButtonMenuWhisper,
				func = function() Whammy:UnitButtonHandleMenu("whisper", unitID) end,
				disabled = nil,
			},
			{
				text = Whammy.localize.kUnitButtonMenuInspect,
				func = function() Whammy:UnitButtonHandleMenu("inspect", unitID) end,
				disabled = isInspectDisabled,
			},
			{
				text = Whammy.localize.kUnitButtonMenuFollow,
				func = function() Whammy:UnitButtonHandleMenu("follow", unitID) end,
				disabled = isFollowDisabled,
			},
			{
				text = Whammy.localize.kUnitButtonMenuCancel,
				func = function() Whammy:UnitButtonHandleMenu("cancel", unitID) end,
				disabled = nil,
			}
		}
	
		EasyMenu( menuList, menuFrame, b, 0, 0, "MENU", 1 )
		
	end
end

function Whammy:UnitButtonHandleMenu( theMenuItem, unitID )
	--Whammy:ShowMessage(theMenuItem)
	if ( theMenuItem == "trade" ) then		
		InitiateTrade( unitID ); --1 = Inspect, 28 yards  2 = Trade, 11.11 yards  3 = Duel, 9.9 yards  4 = Follow, 28 yards
	elseif ( theMenuItem == "whisper" ) then
		local unitName = UnitName( unitID )
		ChatFrame_SendTell( unitName );
	elseif ( theMenuItem == "inspect" ) then
		InspectUnit( unitID )
	elseif ( theMenuItem == "follow" ) then
		FollowUnit( unitID )
	elseif ( theMenuItem == "cancel" )	then
		CloseDropDownMenus();
	end
	
end

--check to see if there is an action corresponding to the mouse click, if not, then show a drop down menu
function Whammy:UnitButtonCheckForAction( b, mouseButton )
	--Whammy:ShowMessage("Checking for action for button "..b:GetName() )
	--mousebutton will be LeftButton, MiddleButton, RightButton, Button4, Button5
	
	local keyPress, mouseButtonNum
	local hasAction;
	
	if ( IsLeftShiftKeyDown() or IsRightShiftKeyDown() ) then
		keyPress = "shift"
	elseif ( IsLeftControlKeyDown() or IsRightControlKeyDown() ) then
		keyPress = "ctrl"
	elseif ( IsLeftAltKeyDown() or IsRightAltKeyDown() ) then
		keyPress = "alt"
	else
		keyPress = ""
	end
	
	if ( keyPress == "ctrl" or keyPress == "shift" or keyPress == "alt") then --if there is a keypress modifier then we need to add 
		keyPress = ( keyPress.."-" ); --                                       the dash so the attribute will be properly formatted
	end
	
	if ( mouseButton == "LeftButton" ) then
		mouseButtonNum = 1
	elseif ( mouseButton == "MiddleButton" ) then
		mouseButtonNum = 3
	elseif ( mouseButton == "RightButton" ) then
		mouseButtonNum = 2
	elseif ( mouseButton == "Button4" ) then
		mouseButtonNum = 4;
	elseif ( mouseButton == "Button5" ) then
		mouseButtonNum = 5;
	end
	
	local x = (keyPress.."type"..mouseButtonNum);
	local a = b:GetAttribute(x)
	
	if ( a and a ~="stop" ) then
		hasAction = true;
		--Whammy:ShowMessage(" button has  action ")
	else
		hasAction = false;
		--Whammy:ShowMessage(" button no action ")
	end
	
	return hasAction;
end

function Whammy:UnitButtonClearThreat() --called when player targets something new, need to reset the threat indicator
	--Whammy:ShowMessage("UnitButtonClearThreat")
	if ( gButtonWithThreat ) then --if someone had threat

		gButtonWithThreat.threatTexture:SetTexture( nil ); --clear their threat
		gButtonWithThreat = nil;
	end
end

-- function Whammy:UnitButtonSetDisconnected( unitID )
-- 	if ( unitID ) then
-- 		local b = Whammy:GetButtonFromUnitID( unitID )
-- 		b.texture:SetTexture( 0,0,0,.2)
-- 	end
-- 
-- end

function Whammy:UnitButtonMouseUp(b) -- unhighlight the name
	if ( b:IsEnabled() ) then
		local unitID = b:GetAttribute("unit"); --get the name of the unit assigned to this button
		if ( unitID ) then
			if ( UnitIsUnit( unitID, "player") ) then --if they are online, color the player one way, and the rest of the units another
				b.unitName:SetFontObject(kFontNamePlayer)
			else --someone else in the party or raid
				b.unitName:SetFontObject(kFontNameOthers)
			end
		end
	end
end

-- End Handle Unit Button Mouse


--
-- MISC functions from here down
--

function Whammy:RangeCheckToggle( enable )
	
	--the function to call every x millisecond to check range of other units
	local onUpdate = function(elapsed)
			Whammy:Event_FrameOnUpdate(elapsed )
		end


	if ( enable == true ) then --turn on range checking
		--Whammy:ShowMessage("Range Checking is Enabled")
		W_Options.RangeCheck = true;
		local playerStatus = Whammy:GetPlayerGroupStatus();--this will return kPlayerAlone, kPlayerInParty or kPlayerInRaid
		
		if ( playerStatus == kPlayerInParty or playerStatus == kPlayerInRaid ) then --if we click to turn on rangeCheck, only do range checking if we are in a party or raid
			whammyFrame:SetScript( "OnUpdate", onUpdate );
		else --not in a party or raid, don't do range checking.
			whammyFrame:SetScript( "OnUpdate", nil);
		end
		
	else -- turn off range checking
		--Whammy:ShowMessage("Range Checking is Disabled")
		W_Options.RangeCheck = false;
		whammyFrame:SetScript( "OnUpdate", nil);
		
		for buttonNum=1,MAX_RAID_MEMBERS do --set the range to off and clear buttons 
			local b = Whammy.buttonArray[buttonNum];
			b.rangeText:SetText("");
		end
	end
	
end



function Whammy:IsUnitIDPlayerPartyRaid( unitID ) --veryify the unitID is formatted partyx or raidxx. Used to eliminate pets from health checking/curse checking
	if ( string.find(unitID, "party%d", 1) or string.find(unitID, "raid%d", 1) or string.find(unitID, "player") ) then
		return true
	else
		return false
	end
end

function Whammy:GetPlayerGroupStatus()
	--this will return kPlayerAlone, kPlayerInParty or kPlayerInRaid
	
	local numGroupMembers = GetNumGroupMembers();
	local playerGroupStatus;
	
	if ( IsInGroup() ) then --player is in some kind of group
		if ( IsInRaid() ) then
			playerGroupStatus = Whammy.constants.kPlayerInRaid;
		else
			playerGroupStatus = Whammy.constants.kPlayerInParty;
		end
	else --player is alone
		playerGroupStatus = Whammy.constants.kPlayerAlone;
	end
	
	return playerGroupStatus
end

function Whammy:GetButtonFromUnitID( unitID )
	local b = Whammy.unitIDArray[unitID];
	
	return b
end

-- Display a message in the chat frame
function Whammy:ShowMessage(aMessage)
	local r,g,b,a
	r = 0;
	g = 1;
	b = 0;
	a = .5;
	
	if ( aMessage ~= nil ) then
		DEFAULT_CHAT_FRAME:AddMessage("Whammy: " ..aMessage, r, g, b, a);
	else
		DEFAULT_CHAT_FRAME:AddMessage("Whammy: nil message");
	end
end



-- handle our one and only slash command. msg is what the user typed i.e. /whammy msg
function Whammy:Slash_Command_Handler(msg)
	--Whammy:ShowMessage("slash handler");

	if ( msg ) then --if the user typed a command then handle it; either reset or config
		if ( msg == "reset" ) then --user wants to reset whammy - reset position, buttons etc.
			Whammy:ShowMessage("Whammy Reset");
			WhammyConfig:Reset_Whammy();
			
		elseif (msg=="show config" or msg=="config" ) then
			WhammyConfig:Config_Frame_Toggle();
			
		else --no command so show/hide whammy frame

			if ( whammyFrame:IsVisible() ) then
				whammyFrame:Hide();
				W_Options.ShowWhammy = false;

			else
				whammyFrame:Show();
				W_Options.ShowWhammy = true;

			end
		end

	end

end


function Whammy:Info_Button_Click() -- show/hide the info frame when the user clicks the ? button
	local frame = WhammyInfoFrame;
	if ( frame ) then
		if ( frame:IsVisible() ) then
			frame:Hide();
		else
			frame:Show();
		end
	end
end

--
-- When the info window loads, set the text that should appear
--
function Whammy:Info_OnLoad(self)
	--Whammy:ShowMessage("Whammy:Info_OnLoad");
	local infoText
	local helpFrameWidth = 440;
	
	WhammyInfoFrame:SetWidth( helpFrameWidth ); --adjust the size of the frame
	WhammyInfoFrame_Heading:SetText("Whammy "..Whammy.version);
	
	-- note that the height of the info frame will adjust around the height of the text. Did this for localization purposes in case another language takes
	--    more or less space to say the same thing
	local t = WhammyInfoFrame_Help_Text;
	
	t:SetPoint("TOPLEFT", "WhammyInfoFrame", 10, -35); --set the starting position of the help text
	t:SetWidth( helpFrameWidth - 15); --set the width of the text
	t:SetText( Whammy.localize.kInfoFrameHelpText ); --set the text from the static text
	local textHeight = t:GetHeight(); --get the height of the text so we can adjust the frame height around it
	WhammyInfoFrame:SetHeight( textHeight + 80 ) --set the frame height to the height of the text + space for the heading area
end




